@@ -2,11 +2,15 @@
  *  Name: dos2unix
  *  Documentation:
  *    Remove cr ('\x0d') characters from a file.
+ *
+ *  The dos2unix package is distributed under FreeBSD style license.
+ *  See also http://www.freebsd.org/copyright/freebsd-license.html
+ *  --------
  * 
- *  Copyright (c) 1994-1995 Benjamin Lin.
- *  Copyright (c) 1998 Bernd Johannes Wuebben
- *  Copyright (c) 1998 Christian Wurll
- *  Copyright (c) 2009-2010 Erwin Waterlander
+ *  Copyright (C) 2009-2010 Erwin Waterlander
+ *  Copyright (C) 1998 Christian Wurll
+ *  Copyright (C) 1998 Bernd Johannes Wuebben
+ *  Copyright (C) 1994-1995 Benjamin Lin.
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -60,16 +64,21 @@
 
 /* #define DEBUG */
 
-#ifdef MSDOS /* DJGPP */
+#if defined(DJGPP) || defined(__TURBOC__) /* DJGPP */
 #  include <dir.h>
 #else
 #  include <libgen.h>
 #endif
+#ifndef __TURBOC__
 #include <unistd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <utime.h>
+#ifdef __TURBOC__
+#define __FLAT__
+#endif
 #include <sys/stat.h>
 #ifdef ENABLE_NLS
 #include <locale.h>
@@ -80,13 +89,16 @@
 #define MSDOS
 #endif
 
-#ifdef MSDOS
+#if  defined(__TURBOC__) || defined(DJGPP) || defined(__MINGW32__)
 /* Some compilers have no mkstemp().
  * Use mktemp() instead.
- * DJGPP, MINGW32 */
+ * BORLANDC, DJGPP, MINGW32 */
 #define NO_MKSTEMP 1
+#endif
+
+#ifdef MSDOS
 /* Some compilers have no fchmod().
- * DJGPP, MINGW32 */
+ * BORLANDC, DJGPP, MINGW32, OPENWATCOM */
 #define NO_FCHMOD 1
 #endif
 
@@ -129,6 +141,7 @@ typedef struct
   int NewLine;                          /* if TRUE, then additional newline */
   int Force;                            /* if TRUE, force conversion of all files. */
   int status;
+  int stdio_mode;                       /* if TRUE, stdio mode */
 } CFlag;
 
 /******************************************************************
@@ -175,10 +188,10 @@ Usage: dos2unix [-fhkLlqV] [-c convmode] [-o file ...] [-n infile outfile ...]\n
 void PrintLicense(void)
 {
   fprintf(stderr, _("\
-Copyright (c) 1994-1995 Benjamin Lin\n\
-Copyright (c) 1998      Bernd Johannes Wuebben (Version 3.0)\n\
-Copyright (c) 1998      Christian Wurll (Version 3.1)\n\
-Copyright (c) 2009-2010 Erwin Waterlander\n\
+Copyright (C) 2009-2010 Erwin Waterlander\n\
+Copyright (C) 1998      Christian Wurll (Version 3.1)\n\
+Copyright (C) 1998      Bernd Johannes Wuebben (Version 3.0)\n\
+Copyright (C) 1994-1995 Benjamin Lin\n\
 All rights reserved.\n\n\
 \
 Redistribution and use in source and binary forms, with or without\n\
@@ -380,6 +393,7 @@ int ConvertDosToUnix(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
               ungetc( TempNextChar, ipInF );  /* put back peek char */
               /* Don't touch this delimiter if it's a CR,LF pair. */
               if ( TempNextChar == '\x0a' ) {
+                putc('\x0d', ipOutF); /* put CR, part of DOS CR-LF */
                 continue;
               }
             }
@@ -406,6 +420,21 @@ int ConvertDosToUnix(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
     return RetVal;
 }
 
+#ifdef __TURBOC__
+char *dirname(char *path)
+{
+  char *ptr;
+
+  if (( path == NULL) || (((ptr=strrchr(path,'/')) == NULL) && ((ptr=strrchr(path,'\\')) == NULL)) )
+    return ".";
+  else
+  {
+    *ptr = '\0';
+    return(path);
+  }
+}
+#endif
+
 #ifdef NO_MKSTEMP
 FILE* MakeTempFileFrom(const char *OutFN, char **fname_ret)
 #else
@@ -593,7 +622,9 @@ int ConvertDosToUnixOldFile(char* ipInFN, CFlag *ipFlag)
   char *TempPath;
   struct stat StatBuf;
   struct utimbuf UTimeBuf;
+#ifndef NO_FCHMOD
   mode_t mode = S_IRUSR | S_IWUSR;
+#endif
 #ifdef NO_MKSTEMP
   FILE* fd;
 #else
@@ -611,8 +642,10 @@ int ConvertDosToUnixOldFile(char* ipInFN, CFlag *ipFlag)
   /* retrieve ipInFN file date stamp */
   if (stat(ipInFN, &StatBuf))
     RetVal = -1;
+#ifndef NO_FCHMOD
   else
     mode = StatBuf.st_mode;
+#endif
 
 #ifdef NO_MKSTEMP
   if((fd = MakeTempFileFrom(ipInFN, &TempPath))==NULL) {
@@ -778,6 +811,7 @@ int main (int argc, char *argv[])
   pFlag->NewLine = 0;
   pFlag->Force = 0;
   pFlag->status = 0;
+  pFlag->stdio_mode = 1;
 
   if ( ((ptr=strrchr(argv[0],'/')) == NULL) && ((ptr=strrchr(argv[0],'\\')) == NULL) )
     ptr = argv[0];
@@ -787,12 +821,6 @@ int main (int argc, char *argv[])
   if ((strcmpi("mac2unix", ptr) == 0) || (strcmpi("mac2unix.exe", ptr) == 0))
     pFlag->ConvMode = 3;
 
-  /* no option, use stdin and stdout */
-  if (argc == 1)
-  {
-    exit(ConvertDosToUnixStdio(pFlag));
-  }
-
   while ((++ArgIdx < argc) && (!ShouldExit))
   {
     /* is it an option? */
@@ -800,26 +828,32 @@ int main (int argc, char *argv[])
     {
       /* an option */
       if ((strcmp(argv[ArgIdx],"-h") == 0) || (strcmp(argv[ArgIdx],"--help") == 0))
+      {
         PrintUsage();
-      if ((strcmp(argv[ArgIdx],"-k") == 0) || (strcmp(argv[ArgIdx],"--keepdate") == 0))
+        return(0);
+      }
+      else if ((strcmp(argv[ArgIdx],"-k") == 0) || (strcmp(argv[ArgIdx],"--keepdate") == 0))
         pFlag->KeepDate = 1;
-      if ((strcmp(argv[ArgIdx],"-f") == 0) || (strcmp(argv[ArgIdx],"--force") == 0))
+      else if ((strcmp(argv[ArgIdx],"-f") == 0) || (strcmp(argv[ArgIdx],"--force") == 0))
         pFlag->Force = 1;
-      if ((strcmp(argv[ArgIdx],"-q") == 0) || (strcmp(argv[ArgIdx],"--quiet") == 0))
+      else if ((strcmp(argv[ArgIdx],"-q") == 0) || (strcmp(argv[ArgIdx],"--quiet") == 0))
         pFlag->Quiet = 1;
-      if ((strcmp(argv[ArgIdx],"-l") == 0) || (strcmp(argv[ArgIdx],"--newline") == 0))
+      else if ((strcmp(argv[ArgIdx],"-l") == 0) || (strcmp(argv[ArgIdx],"--newline") == 0))
         pFlag->NewLine = 1;
-      if ((strcmp(argv[ArgIdx],"-V") == 0) || (strcmp(argv[ArgIdx],"--version") == 0))
+      else if ((strcmp(argv[ArgIdx],"-V") == 0) || (strcmp(argv[ArgIdx],"--version") == 0))
       {
         PrintVersion();
 #ifdef ENABLE_NLS
         PrintLocaledir(localedir);
 #endif
+        return(0);
       }
-      if ((strcmp(argv[ArgIdx],"-L") == 0) || (strcmp(argv[ArgIdx],"--license") == 0))
+      else if ((strcmp(argv[ArgIdx],"-L") == 0) || (strcmp(argv[ArgIdx],"--license") == 0))
+      {
         PrintLicense();
-
-      if ((strcmp(argv[ArgIdx],"-c") == 0) || (strcmp(argv[ArgIdx],"--convmode") == 0))
+        return(0);
+      }
+      else if ((strcmp(argv[ArgIdx],"-c") == 0) || (strcmp(argv[ArgIdx],"--convmode") == 0))
       {
         if (++ArgIdx < argc)
         {
@@ -847,7 +881,7 @@ int main (int argc, char *argv[])
         }
       }
 
-      if ((strcmp(argv[ArgIdx],"-o") == 0) || (strcmp(argv[ArgIdx],"--oldfile") == 0))
+      else if ((strcmp(argv[ArgIdx],"-o") == 0) || (strcmp(argv[ArgIdx],"--oldfile") == 0))
       {
         /* last convert not paired */
         if (!CanSwitchFileMode)
@@ -859,7 +893,7 @@ int main (int argc, char *argv[])
         pFlag->NewFile = 0;
       }
 
-      if ((strcmp(argv[ArgIdx],"-n") == 0) || (strcmp(argv[ArgIdx],"--newfile") == 0))
+      else if ((strcmp(argv[ArgIdx],"-n") == 0) || (strcmp(argv[ArgIdx],"--newfile") == 0))
       {
         /* last convert not paired */
         if (!CanSwitchFileMode)
@@ -870,9 +904,14 @@ int main (int argc, char *argv[])
         }
         pFlag->NewFile = 1;
       }
+      else { /* wrong option */
+        PrintUsage();
+        return(0);
+      }
     }
     else
     {
+      pFlag->stdio_mode = 0;
       /* not an option */
       if (pFlag->NewFile)
       {
@@ -927,6 +966,13 @@ int main (int argc, char *argv[])
     }
   }
 
+  /* no file argument, use stdin and stdout */
+  if (pFlag->stdio_mode)
+  {
+    exit(ConvertDosToUnixStdio(pFlag));
+  }
+
+
   if ((!pFlag->Quiet) && (!CanSwitchFileMode))
   {
     fprintf(stderr, _("dos2unix: target of file %s not specified in new file mode\n"), argv[ArgIdx-1]);
