@@ -4,8 +4,12 @@
  *    Convert lf ('\x0a') characters in a file to cr lf ('\x0d' '\x0a')
  *    combinations.
  *
- *  Copyright (c) 1994-1995 Benjamin Lin.
- *  Copyright (c) 2009-2010 Erwin Waterlander
+ *  The dos2unix package is distributed under FreeBSD style license.
+ *  See also http://www.freebsd.org/copyright/freebsd-license.html
+ *  --------
+ * 
+ *  Copyright (C) 2009-2010 Erwin Waterlander
+ *  Copyright (C) 1994-1995 Benjamin Lin.
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -50,16 +54,21 @@
 
 /* #define DEBUG */
 
-#ifdef MSDOS /* DJGPP */
+#if defined(DJGPP) || defined(__TURBOC__) /* DJGPP */
 #  include <dir.h>
 #else
 #  include <libgen.h>
 #endif
+#ifndef __TURBOC__
 #include <unistd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <utime.h>
+#ifdef __TURBOC__
+#define __FLAT__
+#endif
 #include <sys/stat.h>
 #ifdef ENABLE_NLS
 #include <locale.h>
@@ -70,13 +79,16 @@
 #define MSDOS
 #endif
 
-#ifdef MSDOS
+#if  defined(__TURBOC__) || defined(DJGPP) || defined(__MINGW32__)
 /* Some compilers have no mkstemp().
  * Use mktemp() instead.
- * DJGPP, MINGW32 */
+ * BORLANDC, DJGPP, MINGW32 */
 #define NO_MKSTEMP 1
+#endif
+
+#ifdef MSDOS
 /* Some compilers have no fchmod().
- * DJGPP, MINGW32 */
+ * BORLANDC, DJGPP, MINGW32, OPENWATCOM */
 #define NO_FCHMOD 1
 #endif
 
@@ -119,6 +131,7 @@ typedef struct
   int NewLine;                          /* if TRUE, then additional newline */
   int Force;                            /* if TRUE, force conversion of all files. */
   int status;
+  int stdio_mode;                       /* if TRUE, stdio mode */
 } CFlag;
 
 /******************************************************************
@@ -165,8 +178,8 @@ Usage: unix2dos [-fhkLlqV] [-c convmode] [-o file ...] [-n infile outfile ...]\n
 void PrintLicense(void)
 {
   fprintf(stderr, _("\
-Copyright (c) 1994-1995 Benjamin Lin\n\
-Copyright (c) 2009-2010 Erwin Waterlander\n\
+Copyright (C) 2009-2010 Erwin Waterlander\n\
+Copyright (C) 1994-1995 Benjamin Lin\n\
 All rights reserved.\n\n\
 \
 Redistribution and use in source and binary forms, with or without\n\
@@ -232,7 +245,7 @@ FILE* OpenOutFile(int fd)
 }
 
 
-void AddDOSNewLine(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag, int CurChar, int PrevChar)
+void AddDOSNewLine(FILE* ipOutF, CFlag *ipFlag, int CurChar, int PrevChar)
 {
   if (ipFlag->NewLine) {  /* add additional CR-LF? */
     /* Don't add line ending if it is a DOS line ending. Only in case of Unix line ending. */
@@ -295,7 +308,7 @@ int ConvertUnixToDos(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
                 fprintf(stderr, _("unix2dos: can not write to output file\n"));
               break;
           } else {
-            AddDOSNewLine( ipInF, ipOutF, ipFlag, TempChar, PreviousChar);
+            AddDOSNewLine( ipOutF, ipFlag, TempChar, PreviousChar);
           }
           PreviousChar = TempChar;
         }
@@ -333,7 +346,7 @@ int ConvertUnixToDos(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
                 fprintf(stderr, _("unix2dos: can not write to output file\n"));
               break;
           } else {
-            AddDOSNewLine( ipInF, ipOutF, ipFlag, TempChar, PreviousChar);
+            AddDOSNewLine( ipOutF, ipFlag, TempChar, PreviousChar);
           }
           PreviousChar = TempChar;
         }
@@ -371,7 +384,7 @@ int ConvertUnixToDos(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
                 fprintf(stderr, _("unix2dos: can not write to output file\n"));
               break;
           } else {
-            AddDOSNewLine( ipInF, ipOutF, ipFlag, TempChar, PreviousChar);
+            AddDOSNewLine( ipOutF, ipFlag, TempChar, PreviousChar);
           }
           PreviousChar = TempChar;
         }
@@ -435,6 +448,21 @@ int ConvertUnixToDos(FILE* ipInF, FILE* ipOutF, CFlag *ipFlag)
     return RetVal;
 }
 
+#ifdef __TURBOC__
+char *dirname(char *path)
+{
+  char *ptr;
+
+  if (( path == NULL) || (((ptr=strrchr(path,'/')) == NULL) && ((ptr=strrchr(path,'\\')) == NULL)) )
+    return ".";
+  else
+  {
+    *ptr = '\0';
+    return(path);
+  }
+}
+#endif
+
 #ifdef NO_MKSTEMP
 FILE* MakeTempFileFrom(const char *OutFN, char **fname_ret)
 #else
@@ -622,7 +650,9 @@ int ConvertUnixToDosOldFile(char* ipInFN, CFlag *ipFlag)
   char *TempPath;
   struct stat StatBuf;
   struct utimbuf UTimeBuf;
+#ifndef NO_FCHMOD
   mode_t mode = S_IRUSR | S_IWUSR;
+#endif
 #ifdef NO_MKSTEMP
   FILE* fd;
 #else
@@ -640,8 +670,10 @@ int ConvertUnixToDosOldFile(char* ipInFN, CFlag *ipFlag)
   /* retrieve ipInFN file date stamp */
   if (stat(ipInFN, &StatBuf))
     RetVal = -1;
+#ifndef NO_FCHMOD
   else
     mode = StatBuf.st_mode;
+#endif
 
 #ifdef NO_MKSTEMP
   if((fd = MakeTempFileFrom(ipInFN, &TempPath))==NULL) {
@@ -807,6 +839,7 @@ int main (int argc, char *argv[])
   pFlag->NewLine = 0;
   pFlag->Force = 0;
   pFlag->status = 0;
+  pFlag->stdio_mode = 1;
 
   if ( ((ptr=strrchr(argv[0],'/')) == NULL) && ((ptr=strrchr(argv[0],'\\')) == NULL) )
     ptr = argv[0];
@@ -816,12 +849,6 @@ int main (int argc, char *argv[])
   if ((strcmpi("unix2mac", ptr) == 0) || (strcmpi("unix2mac.exe", ptr) == 0))
     pFlag->ConvMode = 3;
 
-  /* no option, use stdin and stdout */
-  if (argc == 1)
-  {
-    exit(ConvertUnixToDosStdio(pFlag));
-  }
-
   while ((++ArgIdx < argc) && (!ShouldExit))
   {
     /* is it an option? */
@@ -829,26 +856,32 @@ int main (int argc, char *argv[])
     {
       /* an option */
       if ((strcmp(argv[ArgIdx],"-h") == 0) || (strcmp(argv[ArgIdx],"--help") == 0))
+      {
         PrintUsage();
-      if ((strcmp(argv[ArgIdx],"-k") == 0) || (strcmp(argv[ArgIdx],"--keepdate") == 0))
+        return(0);
+      }
+      else if ((strcmp(argv[ArgIdx],"-k") == 0) || (strcmp(argv[ArgIdx],"--keepdate") == 0))
         pFlag->KeepDate = 1;
-      if ((strcmp(argv[ArgIdx],"-f") == 0) || (strcmp(argv[ArgIdx],"--force") == 0))
+      else if ((strcmp(argv[ArgIdx],"-f") == 0) || (strcmp(argv[ArgIdx],"--force") == 0))
         pFlag->Force = 1;
-      if ((strcmp(argv[ArgIdx],"-q") == 0) || (strcmp(argv[ArgIdx],"--quiet") == 0))
+      else if ((strcmp(argv[ArgIdx],"-q") == 0) || (strcmp(argv[ArgIdx],"--quiet") == 0))
         pFlag->Quiet = 1;
-      if ((strcmp(argv[ArgIdx],"-l") == 0) || (strcmp(argv[ArgIdx],"--newline") == 0))
+      else if ((strcmp(argv[ArgIdx],"-l") == 0) || (strcmp(argv[ArgIdx],"--newline") == 0))
         pFlag->NewLine = 1;
-      if ((strcmp(argv[ArgIdx],"-V") == 0) || (strcmp(argv[ArgIdx],"--version") == 0))
+      else if ((strcmp(argv[ArgIdx],"-V") == 0) || (strcmp(argv[ArgIdx],"--version") == 0))
       {
         PrintVersion();
 #ifdef ENABLE_NLS
         PrintLocaledir(localedir);
 #endif
+        return(0);
       }
-      if ((strcmp(argv[ArgIdx],"-L") == 0) || (strcmp(argv[ArgIdx],"--license") == 0))
+      else if ((strcmp(argv[ArgIdx],"-L") == 0) || (strcmp(argv[ArgIdx],"--license") == 0))
+      {
         PrintLicense();
-
-      if ((strcmp(argv[ArgIdx],"-c") == 0) || (strcmp(argv[ArgIdx],"--convmode") == 0))
+        return(0);
+      }
+      else if ((strcmp(argv[ArgIdx],"-c") == 0) || (strcmp(argv[ArgIdx],"--convmode") == 0))
       {
         if (++ArgIdx < argc)
         {
@@ -876,7 +909,7 @@ int main (int argc, char *argv[])
         }
       }
 
-      if ((strcmp(argv[ArgIdx],"-o") == 0) || (strcmp(argv[ArgIdx],"--oldfile") == 0))
+      else if ((strcmp(argv[ArgIdx],"-o") == 0) || (strcmp(argv[ArgIdx],"--oldfile") == 0))
       {
         /* last convert not paired */
         if (!CanSwitchFileMode)
@@ -888,7 +921,7 @@ int main (int argc, char *argv[])
         pFlag->NewFile = 0;
       }
 
-      if ((strcmp(argv[ArgIdx],"-n") == 0) || (strcmp(argv[ArgIdx],"--newfile") == 0))
+      else if ((strcmp(argv[ArgIdx],"-n") == 0) || (strcmp(argv[ArgIdx],"--newfile") == 0))
       {
         /* last convert not paired */
         if (!CanSwitchFileMode)
@@ -899,9 +932,14 @@ int main (int argc, char *argv[])
         }
         pFlag->NewFile = 1;
       }
+      else { /* wrong option */
+        PrintUsage();
+        return(0);
+      }
     }
     else
     {
+      pFlag->stdio_mode = 0;
       /* not an option */
       if (pFlag->NewFile)
       {
@@ -966,6 +1004,13 @@ int main (int argc, char *argv[])
     }
   }
 
+  /* no file argument, use stdin and stdout */
+  if (pFlag->stdio_mode)
+  {
+    exit(ConvertUnixToDosStdio(pFlag));
+  }
+
+
   if ((!pFlag->Quiet) && (!CanSwitchFileMode))
   {
     fprintf(stderr, _("unix2dos: target of file %s not specified in new file mode\n"), argv[ArgIdx-1]);
